
subroutine CalcEnergy(Pos, q, rc, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-3) :: q
    real(8), intent(in) :: rc
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 1.
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d, d2, id2, id6, id12, rc2, Shift, q1, q2
    integer :: i, j
    PEnergy = 0.
    rc2 = rc * rc
    Shift = - (rc**(-12) - rc**(-6))  
    do i = 0, NAtom-1
       Posi = Pos(i,:)
       do j = i + 1, NAtom - 1
         rij = Pos(j,:) - Posi
         d2 = sum(rij*rij)
         d = sqrt(d2)
         if (j == i + 1) then
           PEnergy = PEnergy + k/2. * (d - r0)**2.        
         else
           !this is unbonded
           if (d2 > rc2) then
             cycle
           end if
           id2 = 1. / d2            !inverse squared distance
           id6 = id2 * id2 * id2    !inverse sixth distance
           id12 = id6 * id6         !inverse twelvth distance
           q1 = q(i,0)
           q2 = q(j,0)
           PEnergy = PEnergy + (id12 - id6) + (q1 * q2) * Exp(-d / 10) / d + Shift - (q1 * q2) * Exp(-rc / 10) / rc  
         end if
       enddo
    enddo
end subroutine


subroutine CalcEnergyForces(Pos, M, L, rc, PEnergy, Forces, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Forces
!f2py intent(in, out, inplace) :: Forces
    real(8), parameter :: k = 3000., r0 = 1.
    real(8), dimension(Dim) :: rij, Posi, fij
    real(8) :: d, d2, id2, id6, id12
    real(8) :: rc2, Shift
    integer :: i, j 
    rc2 = rc * rc
    PEnergy = 0.
    Shift = -4. * (rc**(-12) - rc**(-6))
    Forces = 0.
    do i = 0, NAtom-1
       !store Pos(i,:) in a temporary array for faster access in j loop
       Posi = Pos(i,:)
       do j = i + 1, NAtom - 1
         rij = Pos(j,:) - Posi
         rij = rij -L*dnint(rij/L)
         d2 = sum(rij*rij)
         if (j == i + 1 .and. mod(j,M) > 0) then
           !this is a bonded interaction
           d = sqrt(d2)
           PEnergy = PEnergy + k/2. * (d - r0)**2 
           Fij = rij * (k * (d - r0)/d)
           Forces(i,:) = Forces(i,:) + Fij
           Forces(j,:) = Forces(j,:) - Fij       
         else
           !this is unbonded
           if (d2 > rc2) then
             cycle
           end if
           id2 = 1. / d2            !inverse squared distance
           id6 = id2 * id2 * id2    !inverse sixth distance
           id12 = id6 * id6         !inverse twelvth distance
           PEnergy = PEnergy + 4. * (id12 - id6) + Shift 
           Fij = rij * ((-48. * id12 + 24. * id6) * id2)
           Forces(i,:) = Forces(i,:) + Fij
           Forces(j,:) = Forces(j,:) - Fij        
         end if
       enddo
    enddo
end subroutine


subroutine VVIntegrate(Pos, Vel, Accel, M, L, rc, dt, KEnergy, PEnergy, Dim, NAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom
    real(8), intent(in) :: L, rc, dt
    real(8), intent(inout), dimension(0:NAtom-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out,inplace) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, M, L, rc, PEnergy, Accel, Dim, NAtom)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine


subroutine CalcIndEnergy(Pos, M, L, rc, PEnergy, Dim, NAtom, MoveAtom)
    implicit none
    integer, intent(in) :: M, Dim, NAtom, MoveAtom
    real(8), intent(in), dimension(0:NAtom-1, 0:Dim-1) :: Pos
    real(8), intent(in) :: L, rc
    real(8), intent(out) :: PEnergy
    real(8), parameter :: k = 3000., r0 = 1.
    real(8), dimension(Dim) :: rij, Posi
    real(8) :: d, d2, id2, id6, id12, rc2, shift
    integer :: j
    PEnergy = 0.
    Shift = -4. * (rc**(-12) - rc**(-6))  
    rc2 = rc * rc
    Posi = Pos(MoveAtom,:)
    do j = 0, NAtom - 1
      rij = Pos(j,:) - Posi
      rij = rij -L*dnint(rij/L)
      d2 = sum(rij*rij)
      if (abs(MoveAtom - j) == 1 .and. mod(max(MoveAtom, j), M) > 0) then
        d = sqrt(d2)
        PEnergy = PEnergy + k/2. * (d - r0)**2
      else if (j == MoveAtom) then
        cycle
      else
        if (d2 > rc2) then
          cycle
        end if
        id2 = 1. / d2            
        id6 = id2 * id2 * id2  
        id12 = id6 * id6         
        PEnergy = PEnergy + 4. * (id12 - id6) + Shift         
      end if
    enddo
end subroutine